#include <cstdio>
#include "mount.h"
#include "window.h"
#include "pattern.h"

#include <algorithm>
#include <vector>
using namespace std;

// void placeGoodPatterns(vector<Pattern>& patterns, vector<Window>& windows) {
//   vector<bool> used(windows.size(), false);
//   for (size_t i = 0; i < patterns.size(); ++i) {
//     for (size_t j = 0; j < windows.size(); ++j) {
//       if (used[j]) {
//         continue;
//       }
//       if (patterns[i].rows > windows[j].rows || patterns[i].cols > windows[j].cols) {
//         continue;
//       }
//       used[j] = true;
//       windows[j].placePattern(patterns[i], 0, 0);
//       patterns[i].mount = Mount(windows[j].ind, 0, 0);
//       break;
//     }
//   }
// }

int getIntersectingCount(const Pattern& p, const Window& w, int r, int c, int &intersection) {
  int res = 0;
  intersection = -1;
  //fprintf(stderr, "(%d %d),", r, c);
  for (int i = 0; i < p.rows; ++i) {
    for (int j = 0; j < p.cols; ++j) {
      //fprintf(stderr, "%d", (int)p.data[i][j]);
      if (w.data[r + i][j + c] != -1 && p.data[i][j]) {
        if (intersection != -1 && intersection != w.data[r + i][j + c]) {
          return 2;
        }
        intersection = w.data[r + i][j + c];
        res = 1;
      }
    }
    //fprintf(stderr, "\n");
  }
  return res;
}

// void placeOtherPatterns(vector<Pattern>& patterns, vector<Window>& windows) {
//   for (size_t i = 0; i < patterns.size(); ++i) {
//     if (patterns[i].mount.windowInd >= 0) {
//       continue;
//     }
//     bool found = false;
//     for (size_t j = 0; j < windows.size() && !found; ++j) {
//       for (int r = 0; r + patterns[i].rows < windows[j].rows && !found; ++r) {
//         for (int c = 0; c + patterns[i].cols < windows[j].cols && !found; ++c) {
//           if (canPlace(patterns[i], windows[j], r, c)) {
//             windows[j].placePattern(patterns[i], r, c);
//             patterns[i].mount = Mount(windows[j].ind, r, c);
//             found = true;
//           }
//         }
//       }
//     }
//   }
// }

bool comparePatternsByInd(const Pattern& lhs, const Pattern& rhs) {
  return lhs.ind < rhs.ind;
}

void solveWindow(Window& window, vector<Pattern>& patterns) {
  for (size_t i = 0; i < patterns.size(); ++i) {
    if (patterns[i].mount.windowInd >= 0) {
      //fprintf(stderr, "rejecred %d\n", patterns[i].ind);
      continue;
    }
    //fprintf(stderr, "testing %d with window %d\n", patterns[i].ind, window.ind);

    int intersecting = 2;
    int x = -1, y = -1;
    int intersectingInd = -1;
    for (int r = 0; r + patterns[i].rows <= window.rows; ++r) {
      for (int c = 0; c + patterns[i].cols <= window.cols; ++c) {
        int inter = 0;
        int res = getIntersectingCount(patterns[i], window, r, c, inter);
        if (res < intersecting) {
          intersectingInd = inter;
          intersecting = res;
          x = r;
          y = c;
        } else if (res == 1 && intersecting == 1) {
          int score = patterns[intersectingInd].value;
          if (score > patterns[inter].value) {
            intersectingInd = inter;
            x = r;
            y = c;
          }
        }
      }
    }
    //fprintf(stderr, "intersects# %d who %d where (%d,%d)\n", intersecting, intersectingInd, x, y);
    bool shouldMount = !intersecting;
    if (intersecting == 1) {
      int score = patterns[intersectingInd].value;
      if (score < patterns[i].value) {
        shouldMount = true;
        window.unmountPattern(patterns[intersectingInd], x, y);
        patterns[intersectingInd].mount = Mount();
      }
    }
    if (shouldMount) {
      window.placePattern(patterns[i], i, x, y);
      patterns[i].mount = Mount(window.ind, x, y);
    }
  }
}

int main() {
  freopen("decoration.in", "rt", stdin);
  freopen("decoration.out", "wt", stdout);
  int N, M;
  scanf("%d%d", &N, &M);
  vector<Window> windows(N);
  vector<Pattern> patterns(M);

  for (int i = 0; i < N; ++i) {
    int a, b;
    scanf("%d%d", &a, &b);
    windows[i] = Window(a, b);
    windows[i].ind = i;
  }
  sort(windows.begin(), windows.end());

  for (int i = 0; i < M; ++i) {
    int p, q, c;
    scanf("%d%d%d", &p, &q, &c);
    patterns[i] = Pattern(p, q, c);
    patterns[i].ind = i + 1;
    for (int j = 0; j < p; ++j) {
      for (int k = 0; k < q; ++k) {
        char cell;
        scanf("%c", &cell);
        if (cell != '0' && cell != '1') {
          // Probably an EOL or some other shit.
          --k;
          continue;
        }
        if (cell == '1') {
          patterns[i].addCell(Cell(j, k));
        }
      }
    }
    // patterns[i].computeGoodness();
  }

  sort(patterns.begin(), patterns.end());
  for (int i = 0; i < windows.size(); ++i) {
    solveWindow(windows[i], patterns);
  }

  sort(patterns.begin(), patterns.end(), comparePatternsByInd);
  for (int i = 0; i < patterns.size(); ++i) {
    patterns[i].mount.print();
  }
  return 0;
}
