#ifndef PATTERN
#define PATTERN
#pragma once

#include "common.h"
#include "mount.h"
#include <vector>

class Pattern {
public:
  Pattern() : rows(0), cols(0), value(0) {}
  Pattern(int _rows, int _cols, int _value) : rows(_rows), cols(_cols), value(_value) {
    data.resize(rows);
    for (int i = 0; i < rows; ++i) {
      data[i].resize(cols, false);
    }
  }

  void addCell(const Cell& c) {
    cells.push_back(c);
    data[c.first][c.second] = true;
  }

  int getArea() const {
    return this->rows * this->cols;
  }

  const bool operator<(const Pattern& p) const {
    return this->getArea() < p.getArea();
  }

  void computeGoodness() {
    if (!data[0][0]) {
      goodness = 1000000;
      return;
    }

    goodness = 0;
    for (int i = 0; i < rows; ++i) {
      int curr = 0;
      for (int j = 0; j < cols; ++j) {
        if (data[i][j]) {
          goodness += curr;
          curr = 0;
          if (i) {
            break;
          }
        } else {
          ++curr;
        }
      }
    }
  }

  void print() const {
    fprintf(stderr, "%d %d %d %d %d %d\n", ind, goodness, rows, cols, value, (int)cells.size());
  }

  int ind;
  int goodness;
  int rows;
  int cols;
  int value;
  Mount mount;
  std::vector<Cell> cells;
  std::vector<std::vector<bool> > data;
};

#endif
